All platforms.

sci.fork

Host cooperation for forkable SCI runtime values.

Host cooperation for forkable SCI runtime values.
raw docstring

Forkableclj/sprotocol

Values with an application-defined realization in a forked SCI world.

fork-value is called at most once for each identical value during one fork. Return an independent realization to isolate it, return this to share an external resource deliberately, or throw to prohibit the fork.

Values with an application-defined realization in a forked SCI world.

`fork-value` is called at most once for each identical value during one
fork. Return an independent realization to isolate it, return `this` to
share an external resource deliberately, or throw to prohibit the fork.

fork-valueclj/s

(fork-value this)

Return this value's realization in the child world.

Return this value's realization in the child world.
sourceraw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries