sci.impl.execution

The execution-local state shared by world selection and dynamic bindings.

The execution-local state shared by world selection and dynamic bindings.
raw docstring

active-bindingsclj/s

(active-bindings)
(active-bindings state)
source

active-bindings-indexclj/s

source

active-cells-holderclj/s

(active-cells-holder)
(active-cells-holder state)
source

active-cells-holder-indexclj/s

source

active-registryclj/s

(active-registry)
(active-registry state)
source

active-registry-indexclj/s

source

active-worldclj/s

(active-world)
(active-world state)
source

active-world-indexclj/s

source

binding-frameclj/s

(binding-frame)
(binding-frame state)
source

binding-frame-indexclj/s

source

binding-scopeclj/s

(binding-scope)
(binding-scope state)
source

binding-scope-indexclj/s

source

call-with-detached-stateclj/s

(call-with-detached-state f)

Invoke f with an empty execution-local state, restoring the caller's interpreter state afterwards. This is the host boundary used when an SCI program constructs an independent interpreter recursively.

Invoke `f` with an empty execution-local state, restoring the caller's
interpreter state afterwards. This is the host boundary used when an SCI
program constructs an independent interpreter recursively.
sourceraw docstring

current-stateclj/s

(current-state)
source

refresh-active-bindings!clj/s

(refresh-active-bindings! state)
source

scope-bindingsclj/s

(scope-bindings)
(scope-bindings state)
source

scope-bindings-indexclj/s

source

set-active-bindings!clj/s

(set-active-bindings! state value)
source

set-active-cells-holder!clj/s

(set-active-cells-holder! state value)
source

set-active-registry!clj/s

(set-active-registry! state value)
source

set-active-world!clj/s

(set-active-world! state value)
source

set-binding-frame!clj/s

(set-binding-frame! state value)
source

set-binding-scope!clj/s

(set-binding-scope! state value)
source

set-scope-bindings!clj/s

(set-scope-bindings! state value)
source

cljdoc builds & hosts documentation for Clojure/Script libraries