sci.impl.refs

Self-describing mutable references whose state is relative to an SCI world.

Self-describing mutable references whose state is relative to an SCI world.
raw docstring

-delay-realized?clj

(-delay-realized? state)
source

-delay-valueclj

(-delay-value state)
source

alter-meta!*clj/s

(alter-meta!* ref f & args)
source

atomclj/s

(atom x & options)

Create an atom whose value and control state fork with its SCI world.

Create an atom whose value and control state fork with its SCI world.
sourceraw docstring

atom-add-watch!clj/s

(atom-add-watch! home registry control-slot key f)
source

atom-alter-meta!clj/s

(atom-alter-meta! home registry control-slot f args)
source

atom-compare-and-set!clj/s

(atom-compare-and-set! this
                       home
                       registry
                       value-slot
                       control-slot
                       old-value
                       new-value)
source

atom-metaclj/s

(atom-meta home registry control-slot)
source

atom-notify-watches!clj/s

(atom-notify-watches! this control old-value new-value)
(atom-notify-watches! this home registry control-slot old-value new-value)
source

atom-remove-watch!clj/s

(atom-remove-watch! home registry control-slot key)
source

atom-reset!clj/s

(atom-reset! this home registry value-slot control-slot new-value)
source

atom-reset-meta!clj/s

(atom-reset-meta! home registry control-slot m)
source

atom-reset-vals!clj/s

(atom-reset-vals! this home registry value-slot control-slot new-value)
source

atom-set-validator!clj/s

(atom-set-validator! this home registry value-slot control-slot validator)
source

atom-swap!clj/s

(atom-swap! this home registry value-slot control-slot f args)
source

atom-swap-vals!clj/s

(atom-swap-vals! this home registry value-slot control-slot f args)
source

atom-validatorclj/s

(atom-validator home registry control-slot)
source

atom-valueclj/s

(atom-value home registry value-slot)
source

atom-watchesclj/s

(atom-watches home registry control-slot)
source

delay*clj/s

(delay* thunk)

Create a delay whose realization cache follows the active SCI world.

Create a delay whose realization cache follows the active SCI world.
sourceraw docstring

delay-realized?clj/s

(delay-realized? home registry state-slot)
source

delay-valueclj/s

(delay-value home registry state-slot)
source

delay?*clj/s

(delay?* x)
source

deliver*clj

(deliver* p value)
source

force*clj/s

(force* x)
source

get-validator*clj/s

(get-validator* ref)
source

memoize*clj/s

(memoize* f)

Clojure-compatible memoize whose cache follows the active SCI world.

Clojure-compatible memoize whose cache follows the active SCI world.
sourceraw docstring

promise*clj

(promise*)

Create a promise with an independent pending delivery cell per world.

Create a promise with an independent pending delivery cell per world.
sourceraw docstring

promise-realized?clj

(promise-realized? home registry state-slot)
source

promise-valueclj

(promise-value home registry state-slot signal)
source

promise-value-with-timeoutclj

(promise-value-with-timeout home
                            registry
                            state-slot
                            signal
                            timeout-ms
                            timeout-val)
source

reset-meta!*clj/s

(reset-meta!* ref m)
source

sci-atom?clj/s

(sci-atom? x)
source

sci-delay?clj/s

(sci-delay? x)
source

sci-promise?clj

(sci-promise? x)
source

set-validator!*clj/s

(set-validator!* ref validator)
source

cljdoc builds & hosts documentation for Clojure/Script libraries