The execution-local state shared by world selection and dynamic bindings.
The execution-local state shared by world selection and dynamic bindings.
(call-with-detached-state f)Invoke f with an empty execution-local state, restoring the caller's
interpreter state afterwards. This is the host boundary used when an SCI
program constructs an independent interpreter recursively.
Invoke `f` with an empty execution-local state, restoring the caller's interpreter state afterwards. This is the host boundary used when an SCI program constructs an independent interpreter recursively.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |