Self-describing mutable references whose state is relative to an SCI world.
Self-describing mutable references whose state is relative to an SCI world.
(atom x & options)Create an atom whose value and control state fork with its SCI world.
Create an atom whose value and control state fork with its SCI world.
(atom-compare-and-set! this
home
registry
value-slot
control-slot
old-value
new-value)(atom-notify-watches! this control old-value new-value)(atom-notify-watches! this home registry control-slot old-value new-value)(atom-set-validator! this home registry value-slot control-slot validator)(delay* thunk)Create a delay whose realization cache follows the active SCI world.
Create a delay whose realization cache follows the active SCI world.
(memoize* f)Clojure-compatible memoize whose cache follows the active SCI world.
Clojure-compatible memoize whose cache follows the active SCI world.
(promise*)Create a promise with an independent pending delivery cell per world.
Create a promise with an independent pending delivery cell per world.
(promise-value-with-timeout home
registry
state-slot
signal
timeout-ms
timeout-val)cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |